@@ -7,7 +7,7 @@ from json import JSONDecodeError
 from typing import Any, Dict, Optional
 
 from . import utils
-from .constants import FB_MOBILE_BASE_URL
+from .constants import FB_BASE_URL, FB_MOBILE_BASE_URL
 from .fb_types import RawPost, Options, Post, RequestFunction
 
 try:
@@ -45,7 +45,9 @@ class PostExtractor:
         re.IGNORECASE,
     )
     image_regex_lq = re.compile(r"background-image: url\('(.+)'\)")
+    video_thumbnail_regex = re.compile(r"background: url\('(.+)'\)")
     post_url_regex = re.compile(r'/story.php\?story_fbid=')
+    video_post_url_regex = re.compile(r'https://www.facebook.com/.+/videos/.+/(.+)/.+')
 
     shares_and_reactions_regex = re.compile(
         r'<script nonce=.*>.*bigPipe.onPageletArrive\((?P<data>\{.*RelayPrefetchedStreamCache.*\})\);'
@@ -72,6 +74,7 @@ class PostExtractor:
             'time': None,
             'image': None,
             'video': None,
+            'video_thumbnail': None,
             'likes': None,
             'comments': None,
             'shares': None,
@@ -93,6 +96,7 @@ class PostExtractor:
             self.extract_post_url,
             self.extract_link,
             self.extract_video,
+            self.extract_video_thumbnail,
         ]
 
         post = self.make_new_post()
@@ -112,9 +116,14 @@ class PostExtractor:
                 logger.warning("Exception while running %s: %r", method.__name__, ex)
 
         if 'reactions' in self.options:
-            reactions = self.extract_reactions()
+            try:
+                reactions = self.extract_reactions()
+            except Exception as ex:
+                logger.warning("Exception while extracting reactions: %r", ex)
+                reactions = {}
+
             if reactions is None:
-                logger.warning("Could not extract reactions")
+                logger.warning("Extract reactions didn't return anything")
             else:
                 post.update(reactions)
 
@@ -161,9 +170,12 @@ class PostExtractor:
                 else:
                     shared_text.append(node.text)
 
-            text = '\n'.join(itertools.chain(post_text, shared_text))
-            post_text = '\n'.join(post_text)
-            shared_text = '\n'.join(shared_text)
+            # Separation between paragraphs
+            paragraph_separator = '\n\n'
+
+            text = paragraph_separator.join(itertools.chain(post_text, shared_text))
+            post_text = paragraph_separator.join(post_text)
+            shared_text = paragraph_separator.join(shared_text)
 
             return {
                 'text': text,
@@ -175,7 +187,7 @@ class PostExtractor:
 
     # TODO: Add the correct timezone
     def extract_time(self) -> PartialPost:
-        page_insights = self.data_ft['page_insights']
+        page_insights = self.data_ft.get('page_insights', {})
 
         for page in page_insights.values():
             try:
@@ -224,10 +236,18 @@ class PostExtractor:
         elements = self.element.find('header a')
         for element in elements:
             href = element.attrs.get('href', '')
-            match = self.post_url_regex.match(href)
-            if match:
+
+            post_match = self.post_url_regex.match(href)
+            video_post_match = self.video_post_url_regex.match(href)
+
+            if post_match:
                 path = utils.filter_query_params(href, whitelist=query_params)
-                url = utils.urljoin(FB_MOBILE_BASE_URL, path)
+                url = utils.urljoin(FB_BASE_URL, path)
+                return {'post_url': url}
+
+            elif video_post_match:
+                video_post_id = video_post_match.group(1)
+                url = utils.urljoin(FB_BASE_URL, f'watch/?v={video_post_id}')
                 return {'post_url': url}
         return None
 
@@ -340,6 +360,9 @@ class PostExtractor:
             'format': 'best',
             'quiet': True,
         }
+        if 'youtube_dl_verbose' in self.options:
+            ydl_opts.quiet = False
+            
         try:
             post_id = self.post.get('post_id')
             video_page = 'https://www.facebook.com/' + post_id
@@ -350,6 +373,16 @@ class PostExtractor:
             logger.error("Error extracting video with youtube-dl: %r", ex)
         return None
 
+    def extract_video_thumbnail(self):
+        thumbnail_element = self.element.find('i[data-sigil="playInlineVideo"]', first=True)
+        if not thumbnail_element:
+            return None
+        style = thumbnail_element.attrs.get('style', '')
+        match = self.video_thumbnail_regex.search(style)
+        if match:
+            return {'video_thumbnail': utils.decode_css_url(match.groups()[0])}
+        return None
+
     def parse_share_and_reactions(self, html: str):
         bad_jsons = self.shares_and_reactions_regex.findall(html)
         for bad_json in bad_jsons:
@@ -378,7 +411,12 @@ class GroupPostExtractor(PostExtractor):
 
     # TODO: This might need to be aware of the timezone and locale (?)
     def extract_time(self) -> PartialPost:
-        # This is a string with this format 'April 3, 2018 at 8:02 PM'
+        time = super().extract_time()
+        if time is not None:
+            return time
+
+        # In case the previous method didn't work try to parse the date.
+        # This only works for old posts that have this date format 'April 3, 2018 at 8:02 PM'.
         time = self.element.find('abbr', first=True).text
         time = datetime.strptime(time, '%B %d, %Y at %I:%M %p')
         return {
